// File    : mexmain.cpp
// Location: ...\fgtbc3d\
// Purpose : Matlab call C++ function interface
// Author  : Changjiang Yang
//
// $Revision: 0.99 $
// $Date	: Mon Jul  8 18:11:31 EDT 2002 $
 
 
#include "mex.h"
#include "FastGauss.h"


/* the gateway function */
void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[])
//plhs: double *v
//prhs: int dim, double *x, double *u, double *y, double h, int p, int K, double e

{
  
  /*  check for proper number of arguments */
  /* NOTE: You do not need an else statement when using mexErrMsgTxt
     within an if statement, because it will never get to the else
     statement if mexErrMsgTxt is executed. (mexErrMsgTxt breaks you out of
     the MEX-file) */
  // int dim, double *x, double *u, double *y, double h, int p, int K, double e
  if(nrhs != 8) 
    mexErrMsgTxt("Eight inputs required.");
  // double *v
  if(nlhs < 1 | nlhs > 2) 
    mexErrMsgTxt("One or two output required.");

  //////////////////////////////////////////////////////////////
  // Input arguments
  //////////////////////////////////////////////////////////////
  
  //------ the 0st input argument: dim---------------//
  int argu = 0;
  /* check to make sure the input argument is a scalar */
  if( !mxIsDouble(prhs[argu]) || mxIsComplex(prhs[argu]) ||
      mxGetN(prhs[argu])*mxGetM(prhs[argu])!=1 ) {
    mexErrMsgTxt("Input dim must be a scalar.");
  }
  /*  get the scalar input dim */
  int dim = (int) mxGetScalar(prhs[argu]);
  if (dim <= 0)
	  mexErrMsgTxt("Input dim must be a positive number.");

  //----- the first input argument: x--------------//
  argu = 1;
  /*  create a pointer to the input vectors srcs */
  double *x = mxGetPr(prhs[argu]);
  /*  input sources */
  int ds = dim; //mrows
  int ns = mxGetM(prhs[argu]) * mxGetN(prhs[argu]) / dim; //ncols

  //------ the second input argument: u---------------//
  argu = 2;
  /*  create a pointer to the input vectors wts */
  double *u = mxGetPr(prhs[argu]);
  /*  get the dimensions of the input weights */
  int Nu = mxGetN(prhs[argu]);
  int dw = mxGetM(prhs[argu]) * Nu / ns; //mrows
  int nw = ns; //ncols

  //----- the third input argument: y--------------//
  argu = 3;
  /*  create a pointer to the input vectors trgs */
  double *y = mxGetPr(prhs[argu]);
  /*  get the dimensions of the input targets */
  int dt = dim; //mrows
  int mt = mxGetM(prhs[argu]) * mxGetN(prhs[argu]) / dim; //ncols
   
  //----- the fourth input argument: h-------------//
  argu = 4;
  /* check to make sure the input argument is a scalar */
  if( !mxIsDouble(prhs[argu]) || mxIsComplex(prhs[argu]) ||
      mxGetN(prhs[argu])*mxGetM(prhs[argu])!=1 ) {
    mexErrMsgTxt("Input h must be a scalar.");
  }
  /*  get the scalar input h */
  double h = mxGetScalar(prhs[argu]);
  if (h <= 0.0)
	  mexErrMsgTxt("Input h must be a positive number.");

  //----- the fifth input argument: p--------------//
  argu = 5;
  /* check to make sure the input argument is a scalar */
  if( !mxIsDouble(prhs[argu]) || mxIsComplex(prhs[argu]) ||
      mxGetN(prhs[argu])*mxGetM(prhs[argu])!=1 ) {
    mexErrMsgTxt("Input p must be a scalar.");
  }
  /*  get the scalar input p */
  int p = (int) mxGetScalar(prhs[argu]);
  if (p <= 0)
	  mexErrMsgTxt("Input p must be a positive number.");


   //----- the sixth input argument: K--------------//
  argu = 6;
  /* check to make sure the input argument is a scalar */
  if( !mxIsDouble(prhs[argu]) || mxIsComplex(prhs[argu]) ||
      mxGetN(prhs[argu])*mxGetM(prhs[argu])!=1 ) {
    mexErrMsgTxt("Input K must be a scalar.");
  }
  /*  get the scalar input K */
  int K = (int) mxGetScalar(prhs[argu]);
  if (K <= 0)
	  mexErrMsgTxt("Input K must be a positive number.");

   //----- the seventh input argument: e-------------//
  argu = 7;
  /* check to make sure the input argument is a scalar */
  if( !mxIsDouble(prhs[argu]) || mxIsComplex(prhs[argu]) ||
      mxGetN(prhs[argu])*mxGetM(prhs[argu])!=1 ) {
    mexErrMsgTxt("Input e must be a scalar.");
  }
  /*  get the scalar input e */
  double e = mxGetScalar(prhs[argu]);
  if (e <= 0.0)
	  mexErrMsgTxt("Input e must be a positive number.");


  //////////////////////////////////////////////////////////////
  // Output arguments
  //////////////////////////////////////////////////////////////
  /*  set the output pointer to the output result(vector) */
  if(Nu == 1)
      plhs[0] = mxCreateDoubleMatrix(dw*mt,1,mxREAL);
  else
      plhs[0] = mxCreateDoubleMatrix(dw,mt,mxREAL);
  
  /*  create a C pointer to a copy of the output result(vector)*/
  double *v = mxGetPr(plhs[0]);

  /* second output is max radius of clusters */
  double *ptRadius;
  double dummy = 0;
  ptRadius = &dummy;
  if (nlhs==2)
  {
    /*  set the output pointer to the output */
    plhs[1] = mxCreateDoubleMatrix(1,1,mxREAL);
  
    /*  create a C pointer to a copy of the output result(vector)*/
    ptRadius = mxGetPr(plhs[1]);
  }

  // function calls;

  FastGauss* pFGT = new FastGauss(ds, dw, u, x, ns, h, p, K, e, ptRadius);


  pFGT->Compute_v_i(y,mt,v);

  delete pFGT;

  return;
  
}
