function p = flow(s,p,t1,t2)

% p = flow(s,p)
% flow points p with deformation map given in s
% p must be 3*N
% s is the returned value of a call to match function
%
% p = flow(s,p,t1,t2) computes from time steps t1 to t2
% where t1 and t2 are integers between 0 and s.T
% t1=0 means we compute rigid transformation first
%
% p = flow(s,p,-1) is same as p = flow(s,p,s.T,0) (inverse flow)
%
% contact : joan_glaunes@yahoo.fr

kerV = 'exp(argin)';  %% BUILT IN KERNEL FLAG kerV, do not remove this comment
if ~strcmp(s.kerV,kerV)
    error('Matching was not performed with the current built-in kernel.')
end

loadstruct('s')

if ~strcmp(kerV,'exp(argin)') && usefgt
    warning('Current built-in kernel is not gaussian. Fast Gauss Transform disabled')
    usefgt = 0;
end

if nargin == 2
    t1 = 0;
    t2 = T;
end

if nargin == 3 && t1 == -1
        t1 = T;
        t2 = 0;
end

prefix = 'flow step t=';

N = size(p,2);

if t1 == 0
    if optim_verbosemode
        disp('performs affine transformation')
    end
    % direct affine transformation
    p = transmatrix * p + repmat(transvector,1,size(p,2));
    t1 = 1;
end

if t2 == 0
    doinvaffine = 1;
    t2 = 1;
else
    doinvaffine = 0;
end

if t1 > t2
    doinvflow = 1;
    X = X(:,:,T:-1:1);
    mom = - mom(:,:,T:-1:1);
    t1 = T-t1+1;
    t2 = T-t2+1;
else
    doinvflow = 0;
end

dp = zeros(size(p));

if optim_verbosemode && usefgt
    disp('use fgt for flow computation')
end
    
for t = t1:t2-1
    if optim_verbosemode
        if doinvflow
            disp(['flow step t=',num2str(T-t+1),' to ',num2str(T-t)])
        else
            disp(['flow step t=',num2str(t),' to ',num2str(t+1)])
        end
    end
    if usefgt
        dp = fgt(3,X(:,:,t),mom(:,:,t),p,sigmaV(t),order,K,e);
    else
        dp(:) = 0;
        for l = 1:N
            for k = 1:nx
                argin = -( ...
                    (X(1,k,t)-p(1,l))^2 + ...
                    (X(2,k,t)-p(2,l))^2 + ...
                    (X(3,k,t)-p(3,l))^2)/sigmaV2(t);
argout = exp(argin);  %% BUILT IN KERNEL kerV, do not remove this comment
                dp(1,l) = dp(1,l) + argout*mom(1,k,t);
                dp(2,l) = dp(2,l) + argout*mom(2,k,t);
                dp(3,l) = dp(3,l) + argout*mom(3,k,t);
            end
        end
    end
    %%%% BEGIN corrector scheme
    %%%% comment this block to use simple Euler scheme
    if usefgt
        dp = dp + fgt(3,X(:,:,t+1),mom(:,:,t+1),p+tau*dp,sigmaV(t+1),order,K,e);
    else
        ptemp = p + tau * dp;
        for l = 1:N
            for k = 1:nx
                argin = -( ...
                    (X(1,k,t+1)-ptemp(1,l))^2 + ...
                    (X(2,k,t+1)-ptemp(2,l))^2 + ...
                    (X(3,k,t+1)-ptemp(3,l))^2)/sigmaV2(t+1);
argout = exp(argin);  %% BUILT IN KERNEL kerV, do not remove this comment
                dp(1,l) = dp(1,l) + argout*mom(1,k,t+1);
                dp(2,l) = dp(2,l) + argout*mom(2,k,t+1);
                dp(3,l) = dp(3,l) + argout*mom(3,k,t+1);
            end
        end
    end
    dp = dp / 2;
    %%%% END corrector scheme
    p = p + tau * normcoefV(t) * dp;
end

if doinvaffine
    if optim_verbosemode
        disp('performs inverse affine transformation')
    end
    % inverse affine transformation
    p = inv(transmatrix) * (p - repmat(transvector,1,size(p,2)));
end
