function h = plotbyu(byufile,arg2,arg3)

% program to plot a byu surface on the current 3D graph
% usage : plotbyu(byufile)
%         plotbyu(byufile,rgbfile)
%         plotbyu(byufile,color)
%         plotbyu(byufile,'curv')
%         plotbyu(byufile,'curv',color)


fbyu = fopen(byufile,'r');

% read header
ncomponents = fscanf(fbyu,'%d',1);	% number of components
npoints = fscanf(fbyu,'%d',1);		% number of vertices
nfaces = fscanf(fbyu,'%d',1);		% number of faces
nedges = fscanf(fbyu,'%d',1);		% number of edges
fscanf(fbyu,'%d',2*ncomponents);	% components (ignored)

% read data
V = fscanf(fbyu,'%f',[3,npoints]);		% vertices
F = flipud(fscanf(fbyu,'%d',3*nfaces));		% faces

fclose(fbyu);

if nargin > 1
    if ischar(arg2)
        switch arg2
            case 'curv'
                C = meancurv(byufile);
                stdC = std(C);
                mnC = mean(C);
                lim = 1;
                minlim = mean(C)-std(C)*lim;
                maxlim = mean(C)+std(C)*lim;
                C(find(C<minlim)) = minlim;
                C(find(C>maxlim)) = maxlim;
                C = C - min(C);
                C = C / max(C);            
                C = shiftdim(reshape(C(abs(F)),3,prod(size(F))/3),-1);
                if nargin == 3
                    C = [arg3(1)*C;arg3(2)*C;arg3(3)*C];
                else
                    C = [C;C;C];
                end
                C = shiftdim(C,1);
            case 'nocurv'
                if nargin == 3
                    if size(arg3,1) == 3 && size(arg3,2) == 1
                        C = repmat(arg3,1,npoints);
                    elseif size(arg3,1) == 1 && size(arg3,2) == npoints
                        C = repmat(arg3,3,1);
                    else
                        C = arg3;
                    end 
                else
                    % default color is red
                    C = repmat([1;0;0],1,npoints);
                end 
            otherwise
                rgbfile = arg2;
                frgb = fopen(rgbfile,'r');
                C = fscanf(frgb,'%d',[3,npoints]);
                fclose(frgb);
        end
    elseif size(arg2,1) == 3 && size(arg2,2) == 1
        C = repmat(arg2,1,npoints);
    elseif size(arg2,1) == 1 && size(arg2,2) == npoints
        C = repmat(arg2,3,1);
    else
        C = arg2;
    end
else
    % default arg2 is red
    C = repmat([1;0;0],1,npoints);
end 

if min(min(min(C))) < max(max(max(C)))
    C = (C-min(min(min(C))))/(max(max(max(C)))-min(min(min(C))));
end

% plot polygons
hold on
ind = [find(F<0);nedges+1];
dind = diff(ind);

    F = abs(F);
    X = reshape(V(1,F),dind(1),nfaces);
    Y = reshape(V(2,F),dind(1),nfaces);
    Z = reshape(V(3,F),dind(1),nfaces);
    if ndims(C) ~= 3
        C = shiftdim(reshape(C(:,F),3,dind(1),nfaces),1);
    end
    h = patch(X,Y,Z,C);

% anti-bug visualisation
set(gcf,'Renderer','zbuffer')

axis equal
axis off
shading interp
camlight right
material dull

